/*
Gre le paramtrage de l'clairage via l'App Virtuino (Smartphone Android)
*/
//$$$$$$$$$$$$$$$$$$$$$$$$ BIBLIOTHEQUES $$$$$$$$$$$$$$$$$$$$$$$$
#include <Wire.h>	// Communication I2C
#include <DS3232RTC.h>		// Horloge DS3231RTC (retourne temps time_t)  https://github.com/JChristensen/DS3232RTC
#include <VirtuinoCM.h>	// App Virtuino (Smartphone Android) http://iliaslamprou.mysch.gr/virtuino/virtuino.zip
#include <TimeLib.h>		// Gestion des dates et heures https://github.com/PaulStoffregen/Time
#include <Timezone.h>		// Heure locale et dcalage Hiver/t https://github.com/JChristensen/Timezone
#include <EEPROM.h>		// Sauvegarde mmoire morte Arduino

//$$$$$$$$$$$$$$$$$$$$$$$$ FICHIERS $$$$$$$$$$$$$$$$$$$$$$$$
#include "lumiere.h"
#include "sauvegarde.h" // pour rcuprer les variables externes de sauvegarde.cpp
#include "virtuinoConnect.h"
#include "wifi.h"
#include "horloge.h"
#include "ventilation.h"
#include "pin.h"
#include "debug.h"

//$$$$$$$$$$$$$$$$$$$$$$$$ DECLARATIONS $$$$$$$$$$$$$$$$$$$$$$$$
// Afectation de noms aux pins virtuels utiles au programme !!! (Respecter les regroupements de numros) !!!
enum  ePinsVirtuino {
	// ***** Pins Virtuino V[] float
	eTemps = 0, // Date heure actuelle
	// V1  V16 Affichage photoperiode des 4 canaux
	eAffichLedPhoto = 1, // premire led cran photopriode
	eMsgErr_1 = 18,
	// V19 : libre
	eAffDebutPuisPh1 = 20, eAffFinPuisPh1 = 31,// Affichage puissances Conserver les suites de numros pour le prg
	eBoutTousBlanc = 32, eBoutTousBleu = 33,
	eEfface = 34,
	eValidP1 = 35, eValidP2 = 36, eValidP3 = 37, eValidP4 = 38, //Conserver les suites de numros pour le prg
	// V39 : libre
	eAffDebutPuisPh2 = 40, eAffFinPuisPh2 = 51,// Affichage puissances Conserver les suites de numros pour le prg
	eAffPuisReelDebut = 52, eAffPuisReelFin = 63,// 52  63 Affichage des puissances actuelle Conserver les suites de numros pour le prg
	eVentPuissMax = 64,
	// Ventirad
	eVentV0 = 65, eVentV100 = 66, // Affichage Puissance pour ventilateur de 0 ou 100%
	eVentEtat = 67, eVentVit = 68,
	eSliderPuissance = 69,//  Rglage puissance
	eSliderPhoto = 70,//  slection des canaux de couleurs
	eValidPuis = 71, eValidPhoto = 72, eValidVent = 73,
	//	74,  75, libres
	eValidH1 = 76, eValidH2 = 77, eValidH3 = 78, eValidH4 = 79, // heure de photopriode
	eValidVentV0 = 80, eValidVentV100 = 81, // Validation TC pour ventilateur de 0 ou 100%
	eSliderMode = 82,
	eReset = 83, eParamDefaut = 84,
	eBoutDebutBlanc = 85, eBoutFinBlanc = 89, // selection LED blanches 
	eBoutDebutBleu = 90, eBoutFinBleu = 94,	// selection LED bleues 
	eBoutUV = 95, eBoutLune = 96,	// selection LED UV ou Lune 
	eAffDebutPuisPh4 = 97, eAffFinPuisPh4 = 108, // Affichage puissances Conserver les suites de numros pour le prg
	eBoutPuisDebut = 109, // 109  120 	boutons de slections puissances
	eAffDebutPuisPh3 = 121, eAffFinPuisPh3 = 132,// V121  V132  Affichage puissances Conserver les suites de numros pour le prg
};

float varMapPwm;
uint16_t varMapMoyenne;
uint16_t canalPuissActuelle[NB_CANAUX];
uint8_t vCanal = 0;	// variable de boucle sur les canaux d'clairage
uint8_t vCoul = 0;	// variable de boucle sur les couleurs d'clairage
byte vModeEclairage = 1;
//uint8_t etatCanal[NB_CANAUX];	// Etat des canaux de spots pour allumer teindre les Led Virtuino 
uint8_t vSliderCoul = 0;
uint8_t vFlagPhotoOld = 0;
uint8_t vSliderPuiss;
uint8_t vSliderPuissOld = 0;
uint32_t  momentDebutIntervalle;	// Moment du dbut du dcompte de la moyenne des tempratures temps
unsigned long INTERVALLE_AJUST_LUMIERE = 5000;	// Intervalle ajustement puissances durant photopriode 5000 ms = 5s
float heureActuelle = 0;	// float pour simulation rapide
time_t momentActuelSec; // moment actuel format time_t (s depuis 1970)
uint32_t debutJourneeSec;	//	Temps time_t (s depuis 1970)  0h00 de la journe en cours
uint32_t debutJourneeSecondeOld = 900000000;	// Initialisation arbitraire pour dclencher l'analyse du temps
uint32_t debutLeverSeconde[NB_COULEURS]; // conversion de l'heure en seconde sous forme time_t
uint32_t finLeverSeconde[NB_COULEURS];
uint32_t debutCoucherSeconde[NB_COULEURS];
uint32_t finCoucherSeconde[NB_COULEURS];
uint32_t calculPhotoMoment[NB_COULEURS][NB_PHASE + 2];// Moments phases (s UTC) + 2: dernier jour prcdent + premier jour suivant
uint8_t calculPhotoPuissance[NB_PHASE + 2][NB_CANAUX];// puissances 4 phases + 2: dernier jour prcdent + premier jour suivant
uint8_t phasePuissMin[NB_CANAUX];
uint8_t phasePuissMax[NB_CANAUX];

//$$$$$$$$$$$$$$$$$$$$$$$$ SETUP $$$$$$$$$$$$$$$$$$$$$$$$
void initialiserLumiere() {
	// Arduino

	for (byte ind = 0; ind < NB_CANAUX; ind++) {
		pinMode(canalPinPWM[ind], OUTPUT);
		digitalWrite(canalPinPWM[ind], 0);	// Eteint tous les canaux pwm 
	}
	// Virtuino
	V[eMsgErr_1] = 0;
	V[eSliderMode] = 1;	// Positionne la barre de slection du mode clairage Virtuino sur : Auto
	V[eSliderPuissance] = 20;	//  Positionne la barre de slection puissance Virtuino  20%

	V[eValidVentV100] = param.ventilateurPuisV100;	// Temprature pour vitesse du ventilateur 100%
	V[eValidVentV0] = param.ventilateurPuisV0;	// Temprature pour vitesse du ventilateur 100%

	// Lance les fonctions
	eteindreLumiere();	// Eteint les canaux blanc et bleu
	virtuinoMajAffichage();	// affiche les derniers paramtres sur App Virtuino
}

//$$$$$$$$$$$$$$$$$$$$$$$$ FONCTIONS $$$$$$$$$$$$$$$$$$$$$$$$
void gererEclairage() {
	modeUtilisation();		// Actions selon le mode d'clairage choisi sur App Virtuino
	virtuinoMajAffichage();	// affiche les derniers paramtres sur App Virtuino
	resetParametres();		// Reset soft des valeurs utilisateur ou par dfaut
}

void modeUtilisation() {
	vModeEclairage = V[eSliderMode];	// Bouton du mode d'clairage sur Virtuino
	switch (vModeEclairage) {	// selon le mode slectionn
	case 0:	// Mode teint
		VirtuinoEffaceSelectionPuissance();
		eteindreLumiere();
		break;
	case 1:	// Mode Programme auto
		VirtuinoEffaceSelectionPuissance();	// Efface les slection de l'cran
		calculerMomentActuel();	// Appelle la fonction, mise  jour des puissances
		calculerIntervallePhotoperiode();// Dtermine l'intervalle entre ajustement des puissance selon photopriode
		break;
	case 2:	// Mode Manuel
		virtuinoSelectionnerPuissance();
		virtuinoReglerPuissance();
		break;
	case 3:	// Mode Programmation
		virtuinoSelectionnerPuissance();
		virtuinoReglerPuissance();
		virtuinoProgrammer();
		break;
	default:
		//		ajusterLumierePhotoperiode();
		break;
	}
}

void resetParametres() {
	// Soft reset Arduino avec paramtres utilisateur
	if (V[eReset] == 1) {
		resetArduino();
	}
	// Reset des paramtres par dfaut
	if (V[eParamDefaut] == 1) {
		flagParametreDefaut = 1;
		initialiserParametres();
	}
}

void VirtuinoEffaceSelectionPuissance()
{
	for (byte i = eBoutDebutBlanc; i <= eBoutLune; i++) { V[i] = 0; }	// Dsactive du 1er au dernier bouton de slection
}

void virtuinoSelectionnerPuissance() {
	vSliderPuiss = V[eSliderPuissance];
	// Slection des boutons
	if (V[eBoutTousBlanc] == 1) {
		for (byte i = eBoutDebutBlanc; i <= eBoutFinBlanc; i++) {
			V[i] = 1;
		}
	}
	if (V[eBoutTousBleu] == 1) {
		for (byte i = eBoutDebutBleu; i <= eBoutFinBleu; i++) {
			V[i] = 1;
		}
	}
	if (V[eEfface] == 1) {
		for (byte i = eBoutDebutBlanc; i <= eBoutLune; i++) {	// teind toutes les diodes de spots
			V[i] = 0;
		}
		for (vCanal = 0; vCanal < NB_CANAUX; vCanal++) { // Pour chaque canal
			byte valPuissCanal = vCanal + eBoutPuisDebut;
			V[valPuissCanal] = 0;
		}
	}
}

void virtuinoReglerPuissance() {
	// Ajuster la puisssance pour les boutons slectionns
	if (vSliderPuiss != vSliderPuissOld) { // Evite de ralentir le programme
		for (vCanal = 0; vCanal < NB_CANAUX; vCanal++) { // Pour chaque canal
			byte boutSpot = vCanal + eBoutDebutBlanc;
			if (V[boutSpot] == 1) {	// Si le bouton est activ
				byte valPuissCanal = vCanal + eBoutPuisDebut;
				V[valPuissCanal] = V[eSliderPuissance];
				byte vMapSliderPuisPwm = map(V[eSliderPuissance], 0, 100, 0, 255);
				if (V[eSliderMode] == 2) {	// Si on est en mode manuel
					analogWrite(canalPinPWM[vCanal], vMapSliderPuisPwm);	// Ajuste la lumire du spot pwm concern
				}
			}
		}
		vSliderPuissOld = vSliderPuiss;
	}
}

void virtuinoProgrammer() {
	// Validation des puissances d'clairage par canal de spot
	if (V[eValidPuis] == 1) { // si on valide l'enregistrer des puisssances
		int vValidPuis = eValidP1;
		for (int vPuisPhase = 0; vPuisPhase < NB_PHASE; vPuisPhase++) { // pour les bouton 35  38 slectionns
			if (V[vValidPuis + vPuisPhase] == 1) {	// Si le bouton de phase est slectionn
				for (vCanal = 0; vCanal < NB_CANAUX; vCanal++) { // Pour chaque canal
					byte boutSpot = vCanal + eBoutDebutBlanc;
					if (V[boutSpot] == 1) {	// Si le bouton du spot est activ
						byte valPuissCanal = vCanal + eBoutPuisDebut;	// Rcupre la puissance correspondant au canal (bouton)
						param.photoPuissance[vPuisPhase][vCanal] = V[valPuissCanal]; // Enregistre la puissance du canal
						EEPROM.put(ADRESSE_DE_SAUVEGARDE, param);	// Sauvegarde la valeur dans l'eeprom (si des valeurs on chang)
					}
				}
				V[vValidPuis + vPuisPhase] = 0;	// teint le bouton slectionn
			}
		}
	}

	// Prafichage des heures de photoperiode pour la couleur
	if (V[eSliderPhoto] != vFlagPhotoOld) {	// Si le slider couleur evolue
		// Afficahge des parametres actuels pour la couleur
		vSliderCoul = V[eSliderPhoto]; // Slider Virtuino parametre de 0 a 3
		int vValidH = eValidH1;
		for (int vPhase = vValidH - eValidH1; vPhase < NB_PHASE; vPhase++) {
			(V[vPhase + vValidH]) = float(param.photoHeureSec[vSliderCoul][vPhase]) / 3600;
		}
		vFlagPhotoOld = V[eSliderPhoto]; // Pour ne pas prafficher d'autres valeurs
	}
	// Validation des photopriodes depuis Virtuino (4 couleurs et 4 phases prvues)
	if (V[eValidPhoto] == 1) {	// si on valide enregistrer la photopriode
		if (V[eValidH1] < V[eValidH2] && V[eValidH2] < V[eValidH3] && V[eValidH3] < V[eValidH4]) { // test de saisie
			// Rcupration de la couleur modifie : blanc, bleu, UV, lune
			int vSliderCoul = V[eSliderPhoto]; // Slider Virtuino parametre de 0 a 3
			//  enregistrement des heures Virtuino en secondes dans les paramtres
			int vValidH = eValidH1;
			for (int vPhase = vValidH - eValidH1; vPhase < NB_PHASE; vPhase++) {
				param.photoHeureSec[vSliderCoul][vPhase] = float(V[vPhase + vValidH]) * 3600; //  XXXXXXXXXXX suprimer H1
			}
			// sauvegarde des paramtres eeprom si des valeurs on chang
			EEPROM.put(ADRESSE_DE_SAUVEGARDE, param);
			V[eMsgErr_1] = 0;
		}
		else {	// Erreur de saisie
			V[eMsgErr_1] = 1;	// Messsage erreur Virtuino
		}
	}

	// Validation des paramtres du ventirad
	if (V[eValidVent] == 1) {	// si on valide l'enregistrement des paramtres du ventilateur
		if (V[eValidVentV0] < V[eValidVentV100]) {
			param.ventilateurPuisV0 = V[eValidVentV0];	// Temprature pour vitesse du ventilateur 0%
			param.ventilateurPuisV100 = V[eValidVentV100];	// Temprature pour vitesse du ventilateur 100%
			EEPROM.put(ADRESSE_DE_SAUVEGARDE, param);	// Enregistre dans l'eeprom si des valeurs on chang 
			V[eMsgErr_1] = 0;
		}
		else {	// Erreur de saisie
			V[eMsgErr_1] = 1;	// Messsage erreur Virtuino
		}
	}
}

void calculerMomentActuel() {	// pour simulations https://www.timestamp.fr/
	momentActuelSec = RTC.get(); // Rcupre le temps RTC sous forme time_t (sec depuis 1970) avec dcalage horaire.
//	momentActuelSec = 1671645600; // SIMULATION 21/12/19h 
	debutJourneeSec = (momentActuelSec - elapsedSecsToday(momentActuelSec)); // moment du dbut de la journe en sec
//	debutJourneeSec = 1671577200; // SIMULATION 21/12/0h
	
// 	xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx Chaque changement de jour, ajuster heure locale (dcalage et t/hiver) ??????? NON TEST ????????????
	if (debutJourneeSecondeOld != debutJourneeSec){
		initialiserHorloge();	// rinitialise horloge.cpp avec heure locale
		debutJourneeSecondeOld = debutJourneeSec; // Le moment actuel sera mis  jour dans prochaine boucle
	}
// pour chaque couleur, calcul moment sec phase (1,2,3,4 plus dernier jour prcedent(0) et premier jour suivant (5))
	for (int vCoul = 0; vCoul < NB_COULEURS; vCoul++) {
		// moment derniere phase jour prcdent (pour calcul de la pente de puissance)
		calculPhotoMoment[vCoul][0] = debutJourneeSec + param.photoHeureSec[vCoul][NB_PHASE - 1] - 86400;	// 86400 s = 24h
		// moments des phases dans la journe		
		for (int vPhase = 1; vPhase < NB_PHASE + 1; vPhase++) {
			calculPhotoMoment[vCoul][vPhase] = debutJourneeSec + param.photoHeureSec[vCoul][vPhase - 1];
		}
		// moment premiere phase du jour suivant (pour calcul de la pente de puissance)
		calculPhotoMoment[vCoul][NB_PHASE + 1] = debutJourneeSec + param.photoHeureSec[vCoul][0] + 86400;
	}

	// Affectation de la puissance du canal pour chaque phase (4 + avant et aprs),
	for (int vCalcPhase = 0; vCalcPhase < NB_PHASE + 2; vCalcPhase++) {
		for (vCanal = 0; vCanal < NB_CANAUX; vCanal++) {
			// affecte puissance dernire phase  celle jour prcedent
			if (vCalcPhase == 0) {
				calculPhotoPuissance[vCalcPhase][vCanal] = param.photoPuissance[NB_PHASE - 1][vCanal];
			}
			// affecte puissance premire phase  celle jour suivant
			else if (vCalcPhase == NB_PHASE + 1) {
				calculPhotoPuissance[vCalcPhase][vCanal] = param.photoPuissance[0][vCanal];
			}
			// sinon affecte puissance premire phase  celle jour suivant
			else {
				calculPhotoPuissance[vCalcPhase][vCanal] = param.photoPuissance[vCalcPhase - 1][vCanal];
			}

			//					String str;
	//					Serial.print(str + "CalcPhase:" + vCalcPhase + " ");
	//					Serial.print(str + "Canal:" + vCanal + " puiss: ");
	//					Serial.println(calculPhotoPuissance[vCalcPhase][vCanal]); //xxxxxxxxxxxxxxxx
		}
	}
}

void calculerIntervallePhotoperiode() {	// lancer la photopriode  intervalles rguliers non bloquants
	if (momentDebutIntervalle != 0) {	// Si le dcompte d'intervalle a commenc
		unsigned long dureeEcoulee = (millis() - momentDebutIntervalle);	// Variable du calcul d'intervalle entre mesures
		if (dureeEcoulee >= INTERVALLE_AJUST_LUMIERE) {			// si l'intervalle prdfini est atteint	
			momentDebutIntervalle = 0;		// Initialise le dbut d'intervalle	
			ajusterLumierePhotoperiode();	// Appelle la fonction, mise  jour des puissances
		}
		else {							// si l'intervalle est en cours
			dureeEcoulee = 0;			// initialise l'intervalle
		}
	}
	else {	// Si la mesure n'a pas dbut
		ajusterLumierePhotoperiode();	// Appelle la fonction, mise  jour des puissances
		momentDebutIntervalle = millis();	// Dbute le dcompte d'intervalle entre mesures
	}
}

void ajusterLumierePhotoperiode() { // Ajuste la puissance de chaque canal (spot) de chaque couleur
	int vPuissDebutPwm;	// Variable provisoire pour exploiter le mappage
	int vPuissFinPwm;
	int vPuissPwm;
	byte vCoul;
	byte vNumCanal;
	// Pour chaque couleur, dfinit la phase ou se situe le moment actuel, calcule la position et ajuste la puissance de chaque canal (spot)
	for (vCoul = 0; vCoul < NB_COULEURS; vCoul++) { // Pour chacune des 4 couleurs 0=blanc, 1=bleu, 2=UV et 3=Lune
		for (int vPhaseSec = 0; vPhaseSec < NB_PHASE + 2; vPhaseSec++) {
			//si le moment actuel est entre les 2 phases analyses
			if (momentActuelSec >= calculPhotoMoment[vCoul][vPhaseSec] && momentActuelSec < calculPhotoMoment[vCoul][vPhaseSec + 1]) {
				// Calcul de la puissance actuelle pour chaque canal concern (du spot) dans la couleur,
				for (vNumCanal = 0; vNumCanal < NB_CANAUX; vNumCanal++) {
					if (canalCouleur[vNumCanal] == vCoul) {
						vPuissDebutPwm = map(calculPhotoPuissance[vPhaseSec][vNumCanal], 0, 100, 0, 255);	// puissance debut (prcdente)
						vPuissFinPwm = map(calculPhotoPuissance[vPhaseSec + 1][vNumCanal], 0, 100, 0, 255);	// puissance fin (suivante)
						// transpose valeur pwm selon la position dans la phase de photoprode
						varMapPwm = map(momentActuelSec, calculPhotoMoment[vCoul][vPhaseSec], calculPhotoMoment[vCoul][vPhaseSec + 1], vPuissDebutPwm, vPuissFinPwm);
						// Eclaire le canal  la puissance calcule
						analogWrite(canalPinPWM[vNumCanal], varMapPwm);
						// dtermine les donnes pour Virtuino
					//	etatCanal[vNumCanal] = 1; // Dtermine l'tat ON du canal
						canalPuissActuelle[vNumCanal] = round(varMapPwm * 100 / 255); // affecte au canal puissance % actuelle arrondie

						// VERIFIER L'AFFICHAGE Virtuino
						//	String str;
						//	Serial.print(str + "Coul:" + vCoul + " ");
						//	Serial.print(str + "Ph:" + vPhaseSec + " ");
						//	Serial.print(str + "Can:" + vNumCanal + " P ");
						//	Serial.println(canalPuissActuelle[vNumCanal]); 
					}
				}
			}

		}
	}

	// Affichage sur Virtuino de la puissance relle des canaux (variable V[])
	// NOTA cette squence mise dans VirtuinoMajAffichage provoque une erreur mise a zero pour le spot 1 !!
	for (byte numAffLed = eAffPuisReelDebut; numAffLed <= eAffPuisReelFin; numAffLed++) {
		byte vCanal = numAffLed - eAffPuisReelDebut;
		V[numAffLed] = canalPuissActuelle[vCanal];	}
}

void eteindreLumiere() {// Eteint tous les canaux
	for (int vCanal = 0; vCanal <= NB_CANAUX; vCanal++) { // pour chaque pin de canal pwm (voir pin.h)
		digitalWrite(canalPinPWM[vCanal], LOW);	// Eteint tous les canaux pwm 
	}
}

void virtuinoMajAffichage() {

	// Ecran Virtuino 1 : LUMIERE
	V[eTemps] = RTC.get();// Envoie le temps RTC Arduino en ms
	// Affichage valeurs du ventirad
	V[eVentPuissMax] = moyMaxPuissSpots;
	V[eVentVit] = ventilateurVitesse;
	V[eVentV0] = param.ventilateurPuisV0;
	V[eVentV100] = param.ventilateurPuisV100;
	V[eVentEtat] = ventilateurEtat; // passe l'tat du ventilateur pour animer l'icone dynamique

// A SIMPLIFIER boucle (les plages de enum ne se suivent pas)
	// Affichage des puissances par phase (Ph1 = 0) et canal de spots
	for (byte i = eAffDebutPuisPh1; i <= eAffFinPuisPh1; i++) { // de dbut  fin Min
		byte k = i - eAffDebutPuisPh1;	// Definit le premier canal 0  ajuster
		V[i] = param.photoPuissance[0][k]; // rcupre les valeurs de puissance
	}
	for (byte i = eAffDebutPuisPh2; i <= eAffFinPuisPh2; i++) { // de dbut  fin max
		byte k = i - eAffDebutPuisPh2;	// Definit le premier canal 0  ajuster
		V[i] = param.photoPuissance[1][k]; // rcupre les valeurs de puissance
	}

	for (byte i = eAffDebutPuisPh3; i <= eAffFinPuisPh3; i++) { // de dbut  fin Min
		byte k = i - eAffDebutPuisPh3;	// Definit le premier canal 0  ajuster
		V[i] = param.photoPuissance[2][k]; // rcupre les valeurs de puissance
	}

	for (int i = eAffDebutPuisPh4; i <= eAffFinPuisPh4; i++) { // de dbut  fin Min
		int k = i - eAffDebutPuisPh4;	// Definit le premier canal 0  ajuster
		V[i] = param.photoPuissance[3][k]; // rcupre les valeurs de puissance
	}

	// Affichage des photopriodes des canaux blanc, bleu, UV et Lune
	byte t = eAffichLedPhoto;
	for (byte vCoul = 0; vCoul < NB_COULEURS; vCoul++) {// 4 couleurs blancs, bleu, UV, Lune
		for (byte vPhase = 0; vPhase < NB_PHASE; vPhase++) {
			V[t] = param.photoHeureSec[vCoul][vPhase];
			t = t + 1;
		}
	}
}

void vDelay(int delayInMillis) {	// vDelay relance Virtuino.run sans bloquage du programme
	long t = millis() + delayInMillis;
	while (millis() < t) virtuinoRun();
}



