/*
Initialise avec les paramtres par dfaut,
Sauvegarde les paramtres
*/
//$$$$$$$$$$$$$$$$$$$$$$$$ BIBLIOTHEQUES $$$$$$$$$$$$$$$$$$$$$$$$
#include <EEPROM.h>
#include <Arduino.h> // pour Serial...

//$$$$$$$$$$$$$$$$$$$$$$$$ FICHIERS $$$$$$$$$$$$$$$$$$$$$$$$
#include "sauvegarde.h"

//$$$$$$$$$$$$$$$$$$$$$$$$ DECLARATIONS $$$$$$$$$$$$$$$$$$$$$$$$
strParametres param; // Structure 
bool flagParametreDefaut = 0;

// Tableau d'affectations, vite modification programme si changement de pin en panne
uint8_t canalPinPWM[NB_CANAUX] = { 11,3,4,12,6,7,8,9,10,46,45,13 }; // Affecter les pins PWM (pin.h) aux canaux dcompts  partir de 0
uint8_t canalCouleur[NB_CANAUX] = { 0,0,0,0,0,1,1,1,1,1,2,3 }; // Affecter les couleurs aux canaux dcompts  partir de 0 de gauche a droite

//$$$$$$$$$$$$$$$$$$$$$$$$ SETUP $$$$$$$$$$$$$$$$$$$$$$$$

void initialiserParametres() {		// initialise avec eeprom et si vide avec par defaut

	uint32_t lectureMotClef;
	EEPROM.get(ADRESSE_MOT_CLE, lectureMotClef);			// Verifie si eeprom contient des donnes

	if (flagParametreDefaut == 1) {		// Pour rinitialiser  distance avec les parametres par dfaut
		lectureMotClef = !MOT_CLE;		// Mot cl diffrent forcera la rinitialisation
		flagParametreDefaut = 0;		// Le slecteur retrouve l'tat normal
	}
//		lectureMotClef = !MOT_CLE;		// XXXXXX SIMULATION Dcommenter une fois rinitialise parametres par dfaut XXXXXX
	Serial.print("Mot cle lu: "); Serial.println(lectureMotClef);
	Serial.print("Mot cle par defaut: "); Serial.println(MOT_CLE);

	if (lectureMotClef == MOT_CLE) {				// En prsence de donnees utilisateur
		EEPROM.get(ADRESSE_DE_SAUVEGARDE, param);	// On les rcupre

		/*
		for (int vCoul = 0; vCoul < NB_COULEURS; vCoul++) {
			for (int vPhase = 0; vPhase < NB_PHASE; vPhase++) {
				String str;
				Serial.print(str + "coul/phase: " + vCoul + "/" + vPhase + " "); Serial.println((float)param.photoHeureSec[vCoul][vPhase] / (float)3600);
			}			
		}
		*/
	}

	else { // eeprom est vide ou  rinitialiser avec les parametres pas defaut
		// Affectation heures photopriode par dfaut en secondes (h x 3600 s)
		uint32_t PHOTO_COUL_PHASE[NB_COULEURS][NB_PHASE] = {
		{32400,39600,64800,75600},	// Couleur blanc 	 9h,11h,18h,21h
		{30600,39600,64800,76500},	// Couleur bleu		 8.5h,11h,18h,21.25h
		{32400,39600,57600,79200},	// Couleur UV		 9h,11h,16h,22h
		{21600,32400,72000,82800}	// Couleur Lune		 6h,9h,20h,23h
		};
		// Initialisation heures phoropriode par dfaut
		for (int vCoul = 0; vCoul < NB_COULEURS; vCoul++) {
			for (int vPhase = 0; vPhase < NB_PHASE; vPhase++) {
				param.photoHeureSec[vCoul][vPhase] = PHOTO_COUL_PHASE[vCoul][vPhase];
			}
		}

		// Affectation puissances dfaut (0-100%) pour 4 phases et 12 canaux 0  11 (5 Blanc,5 Bleu,UV,Lune)
		uint8_t PUISS_PHASE_CANAUX[NB_PHASE][NB_CANAUX] = {
				{0,0,0,0,0,0,0,0,0,0,0,3},				// Phase 0
				{60,60,60,60,60,50,50,50,50,50,10,0},	// Phase 1
				{50,50,50,50,50,40,40,40,40,40,10,0},	// Phase 2
				{0,0,0,0,0,0,0,0,0,0,0,2},				// Phase 3
		};

		// Initialisation puissances  par dfaut
		for (int vPhase = 0; vPhase < NB_PHASE; vPhase++) {
			for (int vCanal = 0; vCanal < NB_CANAUX; vCanal++) {
				param.photoPuissance[vPhase][vCanal] = PUISS_PHASE_CANAUX[vPhase][vCanal];
			}
		}

		// Initialisation ajustement du temps local avec dclage horaire
		param.tempsLocalEtat = 1;	// Synchronisation du temps local vers RTC
		// Initialisation ventilateur
		param.ventilateurPuisV0 = VENT_PUISS_V0_DEFAUT;
		param.ventilateurPuisV100 = VENT_PUISS_V100_DEFAUT;

		EEPROM.put(ADRESSE_MOT_CLE, MOT_CLE);		// ecrit le mot cle pour signaler que l'eeprom contient des donnes
		EEPROM.put(ADRESSE_DE_SAUVEGARDE, param);	// put : enregistre dans l'eeprom seulement si des valeurs ont chang

	}
}
