/*
* Dtermine la vitesse identique de tous les ventilateurs
* A 0%  une puissance mini prdetermine
* A 100% au del d'une puissance maxi prdtermine
*/
//$$$$$$$$$$$$$$$$$$$$$$$$ BIBLIOTHEQUES $$$$$$$$$$$$$$$$$$$$$$$$
#include <Arduino.h> // pour Serial...

//$$$$$$$$$$$$$$$$$$$$$$$$ FICHIERS $$$$$$$$$$$$$$$$$$$$$$$$
#include "ventilation.h"
#include "lumiere.h"
#include "pin.h"
#include "sauvegarde.h" // pour rcuprer les variables externes de sauvegarde.cpp

//$$$$$$$$$$$$$$$$$$$$$$$$ DECLARATIONS $$$$$$$$$$$$$$$$$$$$$$$$

uint8_t moyennePuissanceSpot[5];	// tableau des moyennes des puissances de lignes de spots
uint8_t moyMaxPuissSpots ;	// valeur maxi qui conditionne la vitesse des ventilateurs 
uint8_t maxPuissanceOld = 0;		// pour agir si volution
uint8_t vitessePwmVentilateur;		// valeur pwm (0-255) des ventilateurs
uint16_t  ventilateurVitesse = 0;		// vitesse 0-100% des ventialteurs
bool flagAjusterVentilation = 1;	// Ajuste la vitesse ventilateurs aprs modification de certains paramtres 
bool ventilateurEtat;

//$$$$$$$$$$$$$$$$$$$$$$$$ SETUP $$$$$$$$$$$$$$$$$$$$$$$$
void initialiserVentilateurs() {
	// Arduino
	pinMode(PIN_PWM_VENTILATEUR, OUTPUT);	// Affecte le mode sortie
	digitalWrite(PIN_PWM_VENTILATEUR, LOW);	// Eteint les ventilateurs
}


//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ LOOP $$$$$$$$$$$$$$$$$$$$

void gererVentilation(void) {
	calculerPuissance();
	ajusterVentilation();
}

// vitesse des ventilateurs (une seule commande pour les 10 ventilateurs)
void calculerPuissance(void) {
	// Calcul de la puissance moyenne des canaux blanc-bleu sur chaque spot
	moyennePuissanceSpot[1] = (canalPuissActuelle[0] + canalPuissActuelle[5]) / 2;	// moyenne spot 1 ligne 0 (blanc) et 5 (bleu)
	moyennePuissanceSpot[2] = (canalPuissActuelle[1] + canalPuissActuelle[6]) / 2;	// Spot 2 ...
	moyennePuissanceSpot[3] = (canalPuissActuelle[2] + canalPuissActuelle[7]) / 2;
	moyennePuissanceSpot[4] = (canalPuissActuelle[3] + canalPuissActuelle[8]) / 2;
	moyennePuissanceSpot[5] = (canalPuissActuelle[4] + canalPuissActuelle[9]) / 2;
	// calcul de la puissance moyenne maximum des spots (blanc plus bleu)
	moyMaxPuissSpots = 0;
		// CALCULER AVEC FONCTION MAX
	for (int i = 1; i <= 5; i++) {	// recherche du maxi pour spots 1  5 (les spots 6  10) sont identiques
		if (moyennePuissanceSpot[i] >= moyMaxPuissSpots) {	// compare chaque spot et retient la valeur suprieure
			//Serial.print("moyennePuissanceSpot[i]: "); Serial.println(moyennePuissanceSpot[i]);
			moyMaxPuissSpots = moyennePuissanceSpot[i];	// Mmorise la puissance maxi
		}
	}
	if (moyMaxPuissSpots != maxPuissanceOld) {
		flagAjusterVentilation = 1;	// Ajustement de la vitesse des ventilateurs
		maxPuissanceOld = moyMaxPuissSpots;
	}

}

void ajusterVentilation(void) {	// variation vitesse ventilateurs	
	if (flagAjusterVentilation == 1) {	// Si volution de paramtres (puissance lumire, seuil de vitesse ventilateurs...)
		if (moyMaxPuissSpots <= param.ventilateurPuisV0) {
			digitalWrite(PIN_PWM_VENTILATEUR, LOW);
			ventilateurVitesse = 0;	// Ventilateurs teints
			ventilateurEtat = 0;	// Etat des ventilateurs OFF		
		}
		else if (moyMaxPuissSpots > param.ventilateurPuisV0 && moyMaxPuissSpots < param.ventilateurPuisV100) {
			vitessePwmVentilateur = map(moyMaxPuissSpots, param.ventilateurPuisV0, param.ventilateurPuisV100, 0, 255);
			analogWrite(PIN_PWM_VENTILATEUR, vitessePwmVentilateur);
			ventilateurVitesse = vitessePwmVentilateur * 100 / 255;	// vitesse 0-100% des ventilateurs
			ventilateurEtat = 1;	// passe l'tat du ventilateur ON

//		Serial.print(" vitessePwmVentilateur: ");  Serial.println(vitessePwmVentilateur);
//		Serial.print(" ventilateurVitesse: ");  Serial.println(ventilateurVitesse);

		}
		else {	// au del de la puissance des spots o la vitesse est  100%
			digitalWrite(PIN_PWM_VENTILATEUR, HIGH);
			ventilateurVitesse = 100;	// Ventilateurs  fond
			ventilateurEtat = 1;// passe l'tat du ventilateur ON
		}
		flagAjusterVentilation = 0;	// Pas d'ajustement prvu
	}
}