/*
Programme de communication avec app smartphone android Virutuino permettant de recevoir et d'envoyer des donnees sur cran tactile
*/
//$$$$$$$$$$$$$$$$$$$$$$$$ BIBLIOTHEQUES $$$$$$$$$$$$$$$$$$$$$$$$
#include <VirtuinoCM.h>// App Virtuino avec WiFi (pour Smartphone Android) http://iliaslamprou.mysch.gr/virtuino/virtuino.zip

//$$$$$$$$$$$$$$$$$$$$$$$$ FICHIERS $$$$$$$$$$$$$$$$$$$$$$$$
#include "virtuinoConnect.h"
#include "wifi.h"
#include "debug.h"

//$$$$$$$$$$$$$$$$$$$$$$$$ DECLARATIONS $$$$$$$$$$$$$$$$$$$$$$$$

VirtuinoCM virtuino;
float V[V_MEMORY_COUNT];    // Tableau synchronis de mmoire virtuelle V de Virtuino. On peut changer le type en int, long etc.

//$$$$$$$$$$$$$$$$$$$$$$$$ SETUP $$$$$$$$$$$$$$$$$$$$$$$$
void initialiserConnexionVirtuino() {
	Serial1.setTimeout(50);	// Serial1 reli  ESP.

	// Dimensionne le tampon (defaut 256) pour contrler 28 pins (1 cde = 9 octets(byte)); 1 cde T(texte) 20 caractres ncessite 20 + 6 octets
	virtuino.begin(onReceived, onRequested, 1280); // Taille du tampon de transfert : 130 pins virtuels soit environ 1170 octets
	// Nota adapter la taille de la variable dans xxxxCM.h ligne 29
	//virtuino.key="1234";    //Mot de passe Virtuino. Only requests the start with this key are accepted from the library
	connectToWiFiNetwork();
}

//$$$$$$$$$$$$$$$$$$$$$$$$ FONCTIONS $$$$$$$$$$$$$$$$$$$$$$$$
// Paramtre l'ESP par des commandes AT en vue d'tablir la connexion rseau
void connectToWiFiNetwork() {
	Serial.println("***** Parametrage ESP8266-01 ");
	Serial.println("Virtuino se connecte a " + String(MY_SSID));
	while (Serial1.available()) Serial1.read();
	Serial1.println("AT+GMR");       // Ecrit les infos du firmware ESP
	waitForResponse("OK", 1000);
	Serial1.println("AT+CWMODE=1");  // configure ESP comme client
	waitForResponse("OK", 1000);
	Serial1.print("AT+CWJAP=\"");    // Connexion au rseau Wifi
	Serial1.print(MY_SSID);
	Serial1.print("\",\"");
	Serial1.print(MY_PASS_WORD);
	Serial1.println("\"");
	waitForResponse("OK", 10000);
	Serial1.print("AT+CIPSTA=\"");		// Affecte une IP  l'ESP
	Serial1.print(ESP_SERVER_IP);
	Serial1.println("\"");
	waitForResponse("OK", 5000);
	Serial1.println("AT+CIPSTA?");		// Demande l'adressse IP
	waitForResponse("OK", 3000);
	Serial1.println("AT+CIFSR");		// Recupre l'adresse IP affecte  l'ESP
	waitForResponse("OK", 1000);
	Serial1.println("AT+CIPMUX=1");		// configure pour connexions multiples   
	waitForResponse("OK", 1000);
	Serial1.println("AT+SLEEP=0");		// DT dsactive la mise en sommeil de l'ESP
	waitForResponse("OK", 1000);		// Attend la prise en compte
//	Serial.print("Mode sommeil : "); Serial.println("AT+SLEEP?");

	Serial1.print("AT+CIPSERVER=1,");	// Cre un serveur, affecte le port
	Serial1.println(ESP_PORT);
	waitForResponse("OK", 1000);
	Serial.println("** Parametrage ESP8266-01 OK");
}

// =========== onReceived : quand la commande est reue
void onReceived(char variableType, uint8_t variableIndex, String valueAsText) {
	if (variableType == 'V') {
		float value = valueAsText.toFloat();        // Convertit le nombre (si format texte) en nombre decimal xxxxx con
		if (variableIndex < V_MEMORY_COUNT) V[variableIndex] = value;    // Copie la valeur reue dans le tableau arduino V memory
	}
}

// =========== onRequested : fonction appelle chaque fois que Virtuino veut lire la valeur d'un pin
String onRequested(char variableType, uint8_t variableIndex) {
	if (variableType == 'V') {
		if (variableIndex < V_MEMORY_COUNT) return  String(V[variableIndex]);   // Retourne la valeur du tableau arduino V memory
	}
	return "";
}

// =========== Attente d'une rponse
boolean waitForResponse(String target1, int timeout) {
	String data = "";
	char a;
	unsigned long startTime = millis();
	boolean rValue = false;
	while (millis() - startTime < timeout) {
		while (Serial1.available() > 0) {
			a = Serial1.read();

			if (DEBUG_VIRTUINO) Serial.print(a);	// Si debug Virtuino activ dans AquaEclairage.ino
			if (a == '\0') continue;
			data += a;
		}
		if (data.indexOf(target1) != -1) {
			rValue = true;
			break;
		}
	}
	return rValue;
}

//$$$$$$$$$$$$$$$$$$$$$$$$ FONCTIONS $$$$$$$$$$$$$$$$$$$$$$$$

void virtuinoRun() {
	if (Serial1.available()) {
		virtuino.readBuffer = Serial1.readStringUntil('\n');
		if (DEBUG_VIRTUINO) Serial.print('\n' + virtuino.readBuffer);	// DEBUG Virtuino  activer dans debug.h
		int pos = virtuino.readBuffer.indexOf("+IPD,");
		if (pos != -1) {
			int connectionId = virtuino.readBuffer.charAt(pos + 5) - 48;  // get connection ID
			int startVirtuinoCommandPos = 1 + virtuino.readBuffer.indexOf(":");
			virtuino.readBuffer.remove(0, startVirtuinoCommandPos);
			String* response = virtuino.getResponse();    // get the text that has to be sent to Virtuino as reply. The library will check the inptuBuffer and it will create the response text
			if (DEBUG_VIRTUINO) Serial.println("\nResponse : " + *response); // DEBUG Virtuino
			if (response->length() > 0) {
				String cipSend = "AT+CIPSEND=";		// ESP envoie les donnes en mode de transmission transparent
				cipSend += connectionId;
				cipSend += ",";
				cipSend += response->length();
				cipSend += "\r\n";
				while (Serial1.available()) Serial1.read();    // clear Serial1 buffer 
				for (int i = 0; i < cipSend.length(); i++) Serial1.write(cipSend.charAt(i));
				if (waitForResponse(">", 1000)) Serial1.print(*response);
				waitForResponse("OK", 1000);
			}
			Serial1.print("AT+CIPCLOSE="); Serial1.println(connectionId); // Ferme la connexion TCP/UDP
		}
	}
}


