/*
 Name:		AquaEclairage.ino
 Created:	07/01/2021 14:29:35
 Author:	Denis Tournassat

 Le programme gre une rampe d'clairage d'aquarium 600W quipe de :
- 1 Arduino Mega 2560
- 5 lignes de 2 spots (10 spots multichips) 2x30W blanc et bleu sur sortie PWM Arduino
- 1 ligne de LED UV 3W sur sortie PWM Arduino
- 1 ligne de LED bleues pour simulation de lune sur sortie PWM Arduino
- 1 puce Wifi ESP-01
- 1 horloge DS3232RTC
- 1 bouton poussoir hard reset,
- des alimentations
- 20 drivers de LED type LDD1000H : 10 blancs, 10 bleus, 1 LDD 700H : UV 1 LDD 350H : Lune

Repartition des canaux (les canaux bleus et blancs sont pilots 2 par 2 de gauche  droite) :
* canal 1  5 (de gauche  droite) : blancs (2 spots)
* canal 6  10 (de gauche  droite) : bleus (2 spots)
* canal 11 : UV
* canal 12 : Lune
- Chaque canal pilot idpendamment en puissance
*/

//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ Bibliothques $$$$$$$$$$$$$$$$$$$$


//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ Fichiers $$$$$$$$$$$$$$$$$$$$
//#include <Arduino.h> // pour Serial...

#include "pin.h"				// Affectation, dclaration des pins
#include "sauvegarde.h"			// Parmtrage par dfaut, sauvegarde des paramtres utilisateur
#include "virtuinoConnect.h"	// Paramtrage ESP, conversion temps, gestion horloge RTC
#include "wifi.h"				// utilisation ESP8266 et connection wifi
#include "horloge.h"			// conversion des temps, gestion horloge RTC
#include "lumiere.h"			// pour exploiter les fonctions dans eclairage.cpp
#include "debug.h"				// activation du dbuggage
#include "ventilation.h"		// activation du dbuggage

//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ Dclarations $$$$$$$$$$$$$$$$$$$

//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ SETUP $$$$$$$$$$$$$$$$$$$$
void setup() {

	if (DEBUG) {	// Active/dsactive le debugage
		Serial.begin(74880);
		while (!Serial) continue;
	}
	Serial.println(); Serial.println("** Debut initialisation");
	initialiserParametres();		// Initialisation cartes PWM ???, paramtrage par dfaut, mise  jour, param
	Serial.println("** Parametres OK");
		initialiserWifiEsp();			// Connexion UDP et serveur de temps NTP
	Serial.println("** Wifi OK");
		initialiserConnexionVirtuino();	// Configure ESP pour connexion Virtuino
	Serial.println("** Virtuino OK");
		initialiserHorloge();			// Conversion: dcalage horaire; heure t; heures, minutes, secondes
	Serial.println("** Horloge OK");
		initialiserVentilateurs();		// gestion ventilateurs selon puissance Leds
	Serial.println("** Ventilateurs OK");
	initialiserLumiere();			// gestion puissance lumire selon photopriode, paramtrages
	Serial.println("** Lumiere OK");
}

//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ LOOP $$$$$$$$$$$$$$$$$$$$

void loop() {
	//	etatWifi(); // ?????????? interet ?
	virtuinoRun();	// Connexion et synchronisation Virtuino <-> Smartphone
	gererVentilation();		// Refroidisement des ventirads
	gererEclairage();		// Programme clairage, affichage Viruino
}