
/*
Recupration de l'heure NTP (epoch) du serveur NTP
Traitement de l'heure NTP en heure Unix (Timestamp) et temps local
exploitation de l'horloge DS3232 RTC
*/
//$$$$$$$$$$$$$$$$$$$$$$$$ BIBLIOTHEQUES $$$$$$$$$$$$$$$$$$$$$$$$
#include <Wire.h>			// Communication I2C
#include <TimeLib.h>		// Gestion des dates et heures https://github.com/PaulStoffregen/Time
#include <Timezone.h>		// Heure locale et dcalage Hiver/t https://github.com/JChristensen/Timezone
#include <DS3232RTC.h>		// Horloge DS3231RTC (retourne temps time_t)  https://github.com/JChristensen/DS3232RTC

//$$$$$$$$$$$$$$$$$$$$$$$$ FICHIERS $$$$$$$$$$$$$$$$$$$$$$$$
#include "horloge.h"
#include "sauvegarde.h"
#include "wifi.h"

//$$$$$$$$$$$$$$$$$$$$$$$$ DECLARATIONS $$$$$$$$$$$$$$$$$$$$$$$$
 //Horloge RTC DS3232 : la bibliotheque DS3232RTC dclare une instance d'objet horloge nomme RTC

// Local time : Date et heure locale (Biblio Time Zone) France (Paris, Lille, Toulouse...)
boolean tempsLocalEtat = 1;	// Etat marche/arrt de l'heure locale automatique
// Nota utilisation de dcalage 0 et 60 par rapport a epoch (UTC locale ???), et non UTC universel (Paris = UTC+1h)
TimeChangeRule franceHIV = { "FrHiv", Last, Sun, Oct, 2, 60 };	// Heure hiver: dernier dimanche octobre  2h00, UTC + 1h
TimeChangeRule franceETE = { "FrEte", Last, Sun, Mar, 3, 120 };// Heure t: dernier dimanche mars  3h00, UTC + 2h
Timezone franceTZ(franceHIV, franceETE);						// Paramtrage de TimeZone France selon Hiver/Et
TimeChangeRule* tcr;											// Pointeur vers rgle changement de temps, pour obtenir TZ
time_t localTime;	// Dclare un objet temps: date heure UTClocale			   
tmElements_t tmRtc;	// Dclaration une structure pour temps en lments (Anne, mois, heure, minute...)

//$$$$$$$$$$$$$$$$$$$$$$$$ SETUP $$$$$$$$$$$$$$$$$$$$$$$$
void initialiserHorloge() {
	// Nota :  la bibliotheque <DS3232RTC.h> initialise dj l'horloge avec RTC.begin();
	setSyncProvider(RTC.get); // synchronise time (Time library) avec RTC. Par defaut 5 mn ? NECESSAIRE si on n'appelle pas l'heure Time ?
	rtcSetTimeZone();	// affecte le temps local
	rtcConvertirTempsHMS();
}

//$$$$$$$$$$$$$$$$$$$$$$$$ FONCTIONS $$$$$$$$$$$$$$$$$$$$$$$$

// Synchronisation de l'horloge RTC avec le temps local  
void rtcSetTimeZone() {
	if (param.tempsLocalEtat == 1) {		//  Etat du calcul du temps local
		localTime = franceTZ.toLocal(epoch, &tcr);	// Affecte l'heure locale et heure d't (Lib Timezone)
		RTC.set(localTime);							// Sauvegarde dans RTC temps local en time_t (sec depuis 1970) avec dcalage horaire
		Serial.println("** Conversion Unix/Local **");
		Serial.print("Temps local >1970 s : ");
		Serial.print(localTime);
		Serial.println("");
	}
}

// Fonction pour verifier le programme. Recupre et crit le temps RTC sous forme heures, minutes, secondes... (Biblio RTC)
void rtcConvertirTempsHMS() {
	RTC.read(tmRtc);				// Lit dans RTC la structure tmRtc (tmElements_t) sous forme HMS
	Serial.print("Tps RTC JMA HMS: ");
	printDigits(tmRtc.Day);			// Appelle la fonction d'criture des valeurs infrieures  10
	Serial.print("/");
	printDigits(tmRtc.Month);
	Serial.print("/");
	Serial.print(tmYearToCalendar(tmRtc.Year));
	Serial.print("  ");						// espace de 3 blocs pour centrer les indications
	printDigits(tmRtc.Hour);
	Serial.print(":");
	printDigits(tmRtc.Minute);
	Serial.print(":");
	printDigits(tmRtc.Second);
	Serial.println();
}

void printDigits(int digits)// Inclut un  "0" devant les unites <10
{
	if (digits < 10)
		Serial.print('0');
	Serial.print(digits);
}
