/*
*/

//$$$$$$$$$$$$$$$$$$$$$$$$ BIBLIOTHEQUES $$$$$$$$$$$$$$$$$$$$$$$$
#include "WiFiEsp.h"		// Connnnexion internet avec ESP8266 sur Arduino https://github.com/bportaluri/WiFiEsp
#include "WiFiEspUdp.h"		// Connexion Udp avec ESP8266 sur Arduino https://github.com/bportaluri/WiFiEsp
#include <TimeLib.h>		// Gestion des dates et heures https://github.com/PaulStoffregen/Time
#include <Timezone.h>		// Heure locale et dcalage Hiver/t https://github.com/JChristensen/Timezone
#include <DS3232RTC.h>		// Horloge DS3232RTC (retourne temps time_t) proche DS1307RTC  https://github.com/JChristensen/DS3232RTC

//$$$$$$$$$$$$$$$$$$$$$$$$ FICHIERS $$$$$$$$$$$$$$$$$$$$$$$$
#include "wifi.h"
#include "sauvegarde.h"
#include "horloge.h"
#include "virtuinoConnect.h"
//$$$$$$$$$$$$$$$$$$$$$$$$ DECLARATIONS $$$$$$$$$$$$$$$$$$$$$$$$
// Dclaration communication Wifi ESP8266-01   # Paramtrage selon quipement #
int status = WL_IDLE_STATUS;			// statut Wifi d'attente de connexion
WiFiEspUDP Udp;							// Instance UDP pour envoi et rception de packets de donnes

// Connection ESP au serveur de temps NTP (Network Time Protocol)
time_t getNtpTime();					// Dclare la fonction (TimeLib.h) synchronisation avec serveur NTP
unsigned long secsSince1900;

// Connection ESP au serveur de temps NTP (Network Time Protocol)
char timeServer[] = "3.fr.pool.ntp.org";	// NTP server francais de temps  UTC local "fr.pool.ntp.org"
unsigned int localPort = 2390;				// Port local ou couter les paquets UDP
byte packetBuffer[NTP_PACKET_SIZE];			// Buffer to hold incoming & outgoing packets
unsigned long epoch;						// Temps type time_t coul depuis le 1/1/1970

//$$$$$$$$$$$$$$$$$$$$$$$$ SETUP $$$$$$$$$$$$$$$$$$$$$$$$
void initialiserWifiEsp() {
	Serial1.begin(115200);		// ouvre le port serie Hardware Serial1 pour ESP (com ESP8266 -> UNO:9600 Bps ; Mega2560:115200 Bps)
	WiFi.init(&Serial1);		// initialise le module ESP Bibliotheque WifiEsp
	connecterUdp();
	connecterNtp();
}

// Connexion ESP svt protocole UDP vers un serveur de temps NTP (Network Time Protocol)
void connecterUdp() {	// Connection Udp pour synchronisation heure NTP

	if (WiFi.status() == WL_NO_SHIELD) {				// Vrifie le prsence du module Wifi
		Serial.println("Module WiFi non detecte ");
		while (true);									// don't continue
	}
	while (status != WL_CONNECTED) {					// Tente de se connecter au rseau Wifi 
		Serial.print("Essai connexion reseau WPA SSID ");
		Serial.println(MY_SSID);
		status = WiFi.begin(MY_SSID, MY_PASS_WORD);		// Connect to WPA/WPA2 network
	}
	Serial.println("Connexion au reseau reussie");	// you're connected now, so print out the data
	Udp.begin(localPort);								// Dbut de communication sur le port local affect
}

// Connexion au serveur de temps NTP (Network Time Protocol) https://fr.wikipedia.org/wiki/Network_Time_Protocol
void connecterNtp() {	// Connection Udp pour synchronisation heure NTP
	setSyncProvider(getNtpTime);		// Appel fonction de synchro temps NTP (Bibliotheque Timelib.h)
	setSyncInterval(SYNC_INTERVAL);		// Determine l'intervale entre les synchros
}

// Reception et exploitation de l'heure NTP locale) >1900 en UTC locale >1970
time_t getNtpTime()	// Lecture des paquets NTP, exemple "TimeNTP" bibliotheque "Time"
{
	while (Udp.parsePacket() > 0);			// supprime les paquets precedemment recus
	Serial.println(" ");
	Serial.println("Requete NTP envoyee ");
	sendNTPpacket(timeServer);				// appelle la fonction pour envoyer la requete NTP vers le serveur de temps UTC local
	uint32_t beginWait = millis();
	while (millis() - beginWait < 2000) {
		int size = Udp.parsePacket();
		if (size >= NTP_PACKET_SIZE) {
			Serial.println("Reponse NTP validee ");
			Udp.read(packetBuffer, NTP_PACKET_SIZE);   // Paquet recu, lecture des donnees vers la memoire
			// convertit quatre bytes de depart a l'emplacement 40 pour un entier long
			secsSince1900 = (unsigned long)packetBuffer[40] << 24;
			secsSince1900 |= (unsigned long)packetBuffer[41] << 16;
			secsSince1900 |= (unsigned long)packetBuffer[42] << 8;
			secsSince1900 |= (unsigned long)packetBuffer[43];

			// ++++++++++++++++++++++
			Serial.println(" ");
			Serial.println("Temps NTP sec >1900: ");	// local ????
			Serial.println(secsSince1900);			// Temps time_t (secondes) NTP local coul depuis le 01/01/1900)

			// conversion en temps Unix = Timestamp
			const unsigned long seventyYears = 2208988800UL;	// 70 ans en secondes
			epoch = secsSince1900 - seventyYears;	//  NTP >1900 - 70 ans = Unix epoch >1970 secondes
			Serial.print("Tps UTC sec >1970: ");
			Serial.println(epoch);					// Temps time_t (secondes) UTC (local ??) coul depuis le 01/01/1970
			// ++++++++++++++++++++++
			return epoch; // Unix time
		}
	}
	Serial.println("Echec de connexion NTP ");
	return 0; // return 0 if unable to get the time	
}

void sendNTPpacket(char* ntpSrv)	// lanc par getNtpTime : Requete de paquets NTP a l'adresse du serveur de temps. Exple "UdpNTPClient" library "WiFiESP"
{
	memset(packetBuffer, 0, NTP_PACKET_SIZE);	// Envoie tous les bits a l'emplacement "0" de la memoire ???????????
	// Initialize values needed to form NTP request (see URL above for details on the packets)
	packetBuffer[0] = 0b11100011;				// LI, Version, Mode
	packetBuffer[1] = 0;						// Stratum, or type of clock
	packetBuffer[2] = 6;						// Polling Interval
	packetBuffer[3] = 0xEC;						// Peer Clock Precision
	// 8 bytes of zero for Root Delay & Root Dispersion
	packetBuffer[12] = 49;
	packetBuffer[13] = 0x4E;
	packetBuffer[14] = 49;
	packetBuffer[15] = 52;

	// Tous les champs NTP ont recu des donnees, maintenant on envoie un paquet de requte d'un "timestamp"
	Udp.beginPacket(ntpSrv, 123); //NTP requests are to port 123
	Udp.write(packetBuffer, NTP_PACKET_SIZE);
	Udp.endPacket();
}
//$$$$$$$$$$$$$$$$$$$$$$$$ FONCTIONS $$$$$$$$$$$$$$$$$$$$$$$$

// reset
void(*resetFunc) (void) = 0;	// soft reset Arduino

void resetArduino() {		// Fonction appelle dans Lumiere.cpp
		resetFunc();		// reboot Arduino
}
/*
// Exemple Hard reset non exploits (pour info)
void reboot(const uint8_t pinHardReset) { // pour faire hard reboot
	digitalWrite(pinHardReset, LOW); // met pin en OFF
	delay(100); // attente
	digitalWrite(pinHardReset, HIGH); // met pin en On
}
reboot(hardResetWIFI); // reboot le module wifi avant initialisation
connecterUdp()

const uint8_t hardResetWIFI = 25; // reset du wifi
*/

/*
void etatWifi() {	// A ACTIVER SI PROBLEME
	if (WiFi.status() != WL_CONNECTED)
	{
		Serial.println("(Re)start WLAN connection");
		WiFi.setLEDs(0, 255, 0); // Red = no connection to anything
		WiFi.begin(WLAN_SSID, WLAN_PASS);
		timeNowWLAN = millis();
		connectionState = WLAN_STARTING_MQTT_DOWN;
	}
	if (WiFi.status() != WL_CONNECTED) {		// pas de connexion Wifi
		Serial.println(" WiFi non connecte ");
		WiFi.begin(WLAN_SSID, WLAN_PASS);
//		connecterUdp();
		connectToWiFiNetwork();

	}
}
*/










